import subprocess
import threading
import datetime
import os

class LliurexMirrorCore:
	
	
	LOCK_FILE="/net/mirror/%s/db/lockfile"
	
	def __init__(self):
		
		self.update_thread=threading.Thread()
		self.percentage=("Got 0%",None)
		self.distro="llx1505"
		
	# init
	
	def startup(self,options):
		
		self.n4d_vars=objects["VariablesManager"]
		self.variable=objects["VariablesManager"].get_variable("LLIUREXMIRROR")
		
		
		if self.variable==None:
			try:
				self.n4d_vars.add_variable("LLIUREXMIRROR",{"internal":{}},"","Lliurex Mirror info variable","lliurex-mirror-core")
			except Exception as e:
				pass
			
		if type(self.variable)!=type({}):
			self.variable={}
			self.variable["internal"]={}
		
	#def init	
	
	
	
	
	def update(self,distro=None):
		
		if distro==None:
			distro=self.distro
	
		lock_file=LliurexMirrorCore.LOCK_FILE%distro
			
			
			
		if self.update_thread.is_alive():
			return [False,"Lliurex-mirror (n4d instance) is running"]
			
		if os.path.exists(lock_file):
			return [False,"Lock file exists"]
		
		self.percentage=("Got 0%",None)
		self.update_thread=threading.Thread(target=self._update,args=(distro,))
		self.update_thread.daemon=True
		self.update_thread.start()
		
		return [True,"Running"]
		
	#def update

	def is_alive(self):
		return self.update_thread.is_alive()
	#def is_alive
	
	def _update(self,distro):
		
		
		p=subprocess.Popen(["/usr/share/n4d/binary-plugins/lliurex_mirror " + distro + " update"],shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)


		try:
			objects["ZCenterVariables"].add_pulsating_color("lliurexmirror")
		except:
			pass

		while True:
			line1=p.stdout.readline().strip("\n")
			if line1!="":
				if "Got " in line1:
					self.percentage=(line1,None)
			else:
				break
				
		self.percentage=(self.percentage[0],p.poll())
		self.set_mirror_info(distro)
		
		try:
			objects["ZCenterVariables"].remove_pulsating_color("lliurexmirror")
		except:
			pass
		
		
	#def _update
	
	def get_progress(self,full=False):
		
		if full:
			return self.percentage
		else:
			return self.percentage[0]
	
	#def get_progress
	
	def get_rc_status(self):
		
		return self.percentage[1]
	
	#def get_rc_status
	
	
	def get_size(self,start_path = '.'):
	
		total_size = 0
		try:
			for dirpath, dirnames, filenames in os.walk(start_path):
				for f in filenames:
					fp = os.path.join(dirpath, f)
					total_size += os.path.getsize(fp)
					
			total_size/=1024*1024*1024.0
			return total_size
		except:
			return 0
	
	#def get_size(start_path = '.'):


	
	def set_mirror_info(self,distro=None):
		
		if distro!=None:
			distro=distro
		else:
			distro=self.distro
		
		
		#self.n4d_vars.set_variable("ZEROCENTERINTERNAL",self.internal_variable)
		
		MIRROR_DATE=datetime.date.today().strftime("%d/%m/%Y")
		MIRROR_SIZE=self.get_size("/net/mirror/"+distro)
		
		self.variable["internal"]["last_mirror_date"]=MIRROR_DATE
		self.variable["internal"]["mirror_size"]=str(MIRROR_SIZE)
		
		print self.n4d_vars.set_variable("LLIUREXMIRROR",self.variable)
		
		#set_custom_text(self,app,text):
		txt="Updated on: " + str(MIRROR_DATE)
		txt+=" # Size: %.2fGB"%MIRROR_SIZE
		try:
			objects["ZCenterVariables"].set_custom_text("lliurexmirror",txt)
			abstract=open('/var/log/lliurex/lliurex-mirror.log','w')
			abstract.write(txt+"\n")
			abstract.close()
		except Exception as e:
			pass
		
		
		
	#def set_mirror_info(self):
	
	
